package cn.com.qmth.examcloud.core.oe.admin.service.bean.exporttask;

import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExportTaskEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExportTaskStatus;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExportTaskType;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExportTaskService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit.ExamAuditQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentQuery;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class ExportTask extends Thread {

    private static final Logger log = LoggerFactory.getLogger(ExportTask.class);

    private static int cacheLockTimeout = 60 * 2;

    private AsyncExportService asyncExportService = SpringContextHolder.getBean(AsyncExportService.class);

    private ExportTaskService exportTaskService = SpringContextHolder.getBean(ExportTaskService.class);

    private RedisClient redisClient = SpringContextHolder.getBean(RedisClient.class);

    @Override
    public void run() {
        for (; ; ) {
            try {
                List<ExportTaskEntity> tasks = exportTaskService.findExportTaskToDisposes();
                if (CollectionUtils.isEmpty(tasks)) {
                    log.info("ExportTask sleep 30s");
                    Util.sleep(30);
                    continue;
                }

                for (ExportTaskEntity task : tasks) {
                    String cacheLock = CacheConstants.LOCK_OE_EXPORT_TASK + task.getId();
                    Boolean tryLock = redisClient.setIfAbsent(cacheLock, cacheLock, cacheLockTimeout);
                    if (!tryLock) {
                        continue;
                    }

                    ExportTaskEntity et = exportTaskService.findById(task.getId());
                    try {
                        if (ExportTaskStatus.WAITING != et.getStatus()) {
                            continue;
                        }

                        exportTaskService.startExportTask(et.getId());
                        if (ExportTaskType.EXAM_DETAIL.equals(et.getType())) {
                            examRecordDetails(et);
                        } else if (ExportTaskType.SCORE_STATISTIC.equals(et.getType())) {
                            examScores(et);
                        } else if (ExportTaskType.EXAM_SCHEDULING.equals(et.getType())) {
                            examScheduling(et);
                        } else if (ExportTaskType.AUDIT.equals(et.getType())) {
                            examAudit(et);
                        }
                        exportTaskService.endExportTask(et.getId());
                    } finally {
                        redisClient.delete(cacheLock);
                    }

                    log.info("ExportTask end, taskId = {}", et.getId());
                }
            } catch (Exception e) {
                log.error("ExportTask error " + e.getMessage(), e);
                Util.sleep(30);
            }
        }
    }

    /**
     * 监考已审
     *
     * @param et
     */
    private void examAudit(ExportTaskEntity et) {
        ExamAuditQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamAuditQuery.class);
        req.setRootOrgId(et.getRootOrgId());
        asyncExportService.asyncExportExamAuditList(et.getId(), req);
    }

    /**
     * 考试明细
     *
     * @param et
     */
    private void examRecordDetails(ExportTaskEntity et) {
        ExamRecordQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamRecordQuery.class);
        req.setRootOrgId(et.getRootOrgId());
        asyncExportService.asyncExportExamRecordDetails(et.getId(), req);
    }

    /**
     * 成绩统计
     *
     * @param et
     */
    private void examScores(ExportTaskEntity et) {
        ExamScoreQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamScoreQuery.class);
        req.setRootOrgId(et.getRootOrgId());
        asyncExportService.asyncExportExamScoreStatistics(et.getId(), req);
    }

    /**
     * 考试进度
     *
     * @param et
     */
    private void examScheduling(ExportTaskEntity et) {
        ExamStudentQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamStudentQuery.class);
        req.setRootOrgId(et.getRootOrgId());
        asyncExportService.asyncExportExamScheduling(et.getId(), req);
    }

}
