package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.MarkResultScoreBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.ClearMarkResultReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.SaveSubjectiveQuestionScoreReq;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.QuestionTypeUtil;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentFinalScoreService;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.question.commons.core.question.QuestionType;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordQuestionsServiceImpl.class);

    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamStudentFinalScoreService examStudentFinalScoreService;

    @Override
    public void clearSubjectiveScore(ClearMarkResultReq req) {
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, req.getExamRecordDataId(),
                ExamRecordDataEntity.class);
        // MongoDB清除小题得分
        if (ExamType.ONLINE == examRecordData.getExamType()
                || ExamType.ONLINE_HOMEWORK == examRecordData.getExamType()) {
            ExamRecordQuestionsEntity eqs = getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordData);
            if (eqs != null && CollectionUtils.isNotEmpty(eqs.getExamQuestionEntities())) {
                for (ExamQuestionEntity eq : eqs.getExamQuestionEntities()) {
                    if (!isObjecttive(eq.getQuestionType())) {
                        eq.setStudentScore(null);
                    }
                }
                examRecordQuestionsRepo.save(eqs);
            }
        }
    }

    private Boolean isObjecttive(QuestionType type) {
        if (QuestionType.SINGLE_CHOICE.equals(type) || QuestionType.MULTIPLE_CHOICE.equals(type)
                || QuestionType.TRUE_OR_FALSE.equals(type)) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public List<ExamQuestionEntity> querySubjectiveAnswerList(Long examRecordDataId) {
        ExamRecordQuestionsEntity examRecordQuestionsEntity = getExamRecordQuestionsAndFixExamRecordDataIfNecessary(
                examRecordDataId);
        List<ExamQuestionEntity> examRecordQuestionList = examRecordQuestionsEntity.getExamQuestionEntities();
        return examRecordQuestionList.stream().filter((obj -> {
            return !QuestionTypeUtil.isObjectiveQuestion(obj.getQuestionType());
        })).collect(Collectors.toList());
    }

    @Override
    public void saveSubjectiveQuestionScore(SaveSubjectiveQuestionScoreReq req) {
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, req.getExamRecordDataId(),
                ExamRecordDataEntity.class);

        // 计算主观题总得分
        double totalSubjectiveScore = 0D;
        for (MarkResultScoreBean scoreInfo : req.getScores()) {
            totalSubjectiveScore += scoreInfo.getScore();
        }

        // 更新ec_oe_exam_score
        ExamScoreEntity examScoreEntity = examScoreRepo.findByExamRecordDataId(req.getExamRecordDataId());
        examScoreEntity.setSubjectiveScore(totalSubjectiveScore);
        double totalScore = totalSubjectiveScore + examScoreEntity.getObjectiveScore();
        examScoreEntity.setTotalScore(totalScore);
        examScoreRepo.save(examScoreEntity);

        // 计算考生的最终分数
        examStudentFinalScoreService.calcAndSaveFinalScore(examRecordData.getExamStudentId());

        // MongoDB保存小题得分
        if (ExamType.ONLINE == examRecordData.getExamType()
                || ExamType.ONLINE_HOMEWORK == examRecordData.getExamType()) {
            ExamRecordQuestionsEntity eqs = getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordData);
            if (eqs != null && CollectionUtils.isNotEmpty(eqs.getExamQuestionEntities())) {
                Map<Integer, Double> scoreMap = req.getScores().stream().collect(Collectors
                        .toMap(MarkResultScoreBean::getOrders, MarkResultScoreBean::getScore, (key1, key2) -> key2));
                for (ExamQuestionEntity eq : eqs.getExamQuestionEntities()) {
                    eq.setStudentScore(scoreMap.get(eq.getOrder()));
                }
                examRecordQuestionsRepo.save(eqs);
            }
        }
    }

    @Override
    public Integer calculationSubjectiveAnswerLength(Long examRecordDataId) {
        ExamRecordQuestionsEntity examRecordQuesitonsEntity = getExamRecordQuestionsAndFixExamRecordDataIfNecessary(
                examRecordDataId);
        if (examRecordQuesitonsEntity == null || examRecordQuesitonsEntity.getExamQuestionEntities() == null
                || examRecordQuesitonsEntity.getExamQuestionEntities().isEmpty()) {
            return 0;
        }
        List<ExamQuestionEntity> examQuestionList = examRecordQuesitonsEntity.getExamQuestionEntities();

        int answerLength = 0;
        for (ExamQuestionEntity examQuestionEntity : examQuestionList) {
            if (!QuestionTypeUtil.isObjectiveQuestion(examQuestionEntity.getQuestionType())
                    && examQuestionEntity.getStudentAnswer() != null) {
                answerLength += examQuestionEntity.getStudentAnswer().length();
            }
        }
        return answerLength;
    }

    @Override
    public ExamRecordQuestionsEntity createExamRecordQuestions(Long examRecordDataId, DefaultPaper defaultPaper) {
        ExamRecordQuestionsEntity examRecordQuestionsEntity = new ExamRecordQuestionsEntity();

        List<ExamQuestionEntity> examQuestionEntityList = new ArrayList<ExamQuestionEntity>();
        List<DefaultQuestionGroup> defaultQuestionGroups = defaultPaper.getQuestionGroupList();
        int order = 0;
        for (int i = 0; i < defaultQuestionGroups.size(); i++) {
            DefaultQuestionGroup defaultQuestionGroup = defaultQuestionGroups.get(i);
            List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrappers = defaultQuestionGroup
                    .getQuestionWrapperList();
            for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrappers) {
                List<DefaultQuestionUnitWrapper> questionUnitWrapperList = defaultQuestionStructureWrapper
                        .getQuestionUnitWrapperList();
                for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : questionUnitWrapperList) {
                    ExamQuestionEntity examQuestionEntity = new ExamQuestionEntity();
                    examQuestionEntity.setExamRecordDataId(examRecordDataId);
                    examQuestionEntity.setMainNumber(i + 1);
                    examQuestionEntity.setOrder(++order);
                    examQuestionEntity.setQuestionId(defaultQuestionStructureWrapper.getQuestionId());
                    examQuestionEntity.setQuestionScore(defaultQuestionUnitWrapper.getQuestionScore());
                    examQuestionEntity.setQuestionType(defaultQuestionUnitWrapper.getQuestionType());
                    examQuestionEntity.setOptionPermutation(defaultQuestionUnitWrapper.getOptionPermutation());
                    examQuestionEntity.setAudioPlayTimes(null);
                    examQuestionEntity.setAnswerType(defaultQuestionUnitWrapper.getAnswerType());

                    examQuestionEntityList.add(examQuestionEntity);
                }
            }
        }

        examRecordQuestionsEntity.setExamQuestionEntities(examQuestionEntityList);
        examRecordQuestionsEntity.setExamRecordDataId(examRecordDataId);
        examRecordQuestionsEntity.setCreationTime(new Date());
        ExamRecordQuestionsEntity saveResult = examRecordQuestionsRepo.save(examRecordQuestionsEntity);
        // redisTemplate.opsForList().leftPushAll(examQuestionKeyPrefix+examRecordDataId,examQuestionEntityList);
        return saveResult;
    }

    /**
     * 获取考试作答记录并修复考试记录数据(如有必要)
     *
     * @param examRecordData
     *            考试记录
     * @return
     */
    @Override
    public ExamRecordQuestionsEntity getExamRecordQuestionsAndFixExamRecordDataIfNecessary(
            ExamRecordDataEntity examRecordData) {
        if (examRecordData == null) {
            throw new StatusException("201101", "examRecordData参数不允许为空");
        }

        Long examRecordDataId = examRecordData.getId();
        String examRecordQuestionId = examRecordData.getExamRecordQuestionsId();

        ExamRecordQuestionsEntity examRecordQuestions;
        if (StringUtils.isNotEmpty(examRecordQuestionId)) {
            // 如果考试作答记录id不为空，则根据id查询考试作答记录
            examRecordQuestions = GlobalHelper.getEntity(examRecordQuestionsRepo, examRecordQuestionId,
                    ExamRecordQuestionsEntity.class);
        } else {
            // 如果考试作答记录id为空，则根据考试记录id获取考试作答记录，并将考试作答记录id保存至examRecordData表中
            examRecordQuestions = examRecordQuestionsRepo.findByExamRecordDataId(examRecordDataId);

            if (examRecordQuestions == null) {
                log.warn("examRecordQuestions is null, examRecordDataId = {} examType:{}", examRecordData.getId(),
                        examRecordData.getExamType());
                throw new StatusException("201101", "examRecordQuestions为空");
            }

            // 将考试作答记录id保存至examRecordData表中,目的：纠正历史数据
            examRecordData.setExamRecordQuestionsId(examRecordQuestions.getId());
            examRecordDataRepo.updateExamRecordDataQuestionIdById(examRecordQuestions.getId(), examRecordData.getId());
        }

        // List<ExamQuestionEntity> examQuestions =
        // examRecordQuestions.getExamQuestionEntities();
        // for (ExamQuestionEntity examQuestion : examQuestions) {
        // if (QuestionType.SINGLE_CHOICE == examQuestion.getQuestionType() ||
        // QuestionType.MULTIPLE_CHOICE == examQuestion.getQuestionType()) {
        // // 选择题 - 转换为新答案格式（兼容旧格式）
        // examQuestion.setCorrectAnswer(QuestionOptionHelper.parseNumbers(examQuestion.getCorrectAnswer()));
        // examQuestion.setStudentAnswer(QuestionOptionHelper.parseNumbers(examQuestion.getStudentAnswer()));
        // }
        // }

        return examRecordQuestions;
    }

    @Override
    public ExamRecordQuestionsEntity getExamRecordQuestionsAndFixExamRecordDataIfNecessary(Long examRecordDataId) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");

        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId,
                ExamRecordDataEntity.class);
        return getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordData);
    }

}
